from pathlib import Path
import json
import os
import string

import click


def flip_char(char: str):
    if char.lower() not in string.ascii_letters:
        return char

    if char in string.ascii_lowercase:
        return string.ascii_uppercase[string.ascii_lowercase.find(char)]
    else:
        return string.ascii_lowercase[string.ascii_uppercase.find(char)]


base_folder = Path(Path.home() / ".config")
config_file = base_folder / "utils_config.json"


def load_config() -> dict:
    """Try to load the computer-specific utils JSON file."""
    if not base_folder.exists():
        os.mkdir(base_folder)

    if not config_file.exists():
        with open(config_file, "w") as f:
            f.write(json.dumps({}, indent=2))
        return {}

    try:
        with open(config_file, "r") as f:
            return json.load(f)
    except json.JSONDecodeError:
        click.echo(
            f"Cannot load config -- potentially corrupt file. Check {config_file:s}"
        )
        return {}


def write_config(config_obj) -> None:
    load_config()  # make sure everything exists
    with open(config_file, "w") as f:
        f.write(json.dumps(config_obj, indent=2))
