import click
import httpx
import time

import bs4
from markdownify import markdownify as md

from src.helpers import load_config, write_config


BASE_URL = "http://localhost:"


def doublecheck_port_number(port: int):
    try:
        if (
            httpx.get(f"http://localhost:{port}/ping").content.decode()
            == "JoplinClipperServer"
        ):
            return True
    except Exception:
        return False


def auth_with_joplin():
    config = load_config()
    if not config.get("JOPLIN_PORT"):
        while True:
            config["JOPLIN_PORT"] = click.prompt(
                "What's the port number of the Joplin Web Clipper service?",
                type=int,
                default=41184,
            )
            config["JOPLIN_PORT"] = str(config["JOPLIN_PORT"])
            if doublecheck_port_number(config["JOPLIN_PORT"]):
                break
            else:
                click.echo("Something's not right there. Is the service enabled?")

    # this triggers the dialog in Joplin to accept the auth connection.
    auth_token = (
        httpx.post(BASE_URL + config["JOPLIN_PORT"] + "/auth").json().get("auth_token")
    )
    click.echo("Check Joplin to allow this connection.")

    while True:
        check_resp = httpx.get(
            BASE_URL + config["JOPLIN_PORT"] + f"/auth/check?auth_token={auth_token}"
        ).json()
        if check_resp.get("status") == "waiting":
            time.sleep(0.5)
        else:
            config["JOPLIN_TOKEN"] = check_resp.get("token")
            break

    write_config(config)


def get_folders():
    c = load_config()
    first_call_success = False
    try:
        resp = httpx.get(
            BASE_URL
            + c.get("JOPLIN_PORT", "")
            + f"/folders?token={c.get('JOPLIN_TOKEN')}"
        )
        if resp.status_code == 403:
            click.echo("got 403")
            auth_with_joplin()
        else:
            resp = resp.json()
            first_call_success = True
    except httpx._exceptions.ConnectError:
        auth_with_joplin()

    if not first_call_success:
        c = load_config()
        resp = httpx.get(
            BASE_URL
            + c.get("JOPLIN_PORT", "")
            + f"/folders?token={c.get('JOPLIN_TOKEN')}"
        ).json()
    return resp


def process_joplin_posts():
    # this will handle the initial auth flow if we aren't already authed
    get_folders()
    c = load_config()
    joplin = BASE_URL + c["JOPLIN_PORT"]
    resp = httpx.get(
        joplin
        + f"/search?query=convert&fields=id,parent_id&token={c.get('JOPLIN_TOKEN')}"
    )
    resp = resp.json()
    if not resp.get("items"):
        click.echo("No notes found with title 'convert'.")
        return
    else:
        items = resp.get("items")

    for blob in items:
        url = (
            httpx.get(
                joplin
                + f"/notes/{blob['id']}?fields=body&token={c.get('JOPLIN_TOKEN')}"
            )
            .json()["body"]
            .strip()
        )
        click.echo(f"Processing {url}...")
        site = httpx.get(url, follow_redirects=True)
        soup = bs4.BeautifulSoup(site.content, features="html5lib")
        # clean up that schizz
        title = soup.title.text
        [t.extract() for t in soup(["script", "head", "style"])]
        body = md(str(soup))
        httpx.put(
            joplin + f"/notes/{blob['id']}?token={c.get('JOPLIN_TOKEN')}",
            json={"title": title, "body": body},
        )

    click.echo(resp)
